﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Reflection;

namespace LiveCodePictureBox
{
    public partial class HauptfensterProxy : Form
    {
        CompilerParameters compilerParams;
        bool bAktualisierungLäuft;
        Assembly compiledAssembly;
        CSharpCodeProvider codeProvider;

        public HauptfensterProxy()
        {
            InitializeComponent();

            bAktualisierungLäuft = false;

            // create the compiler
            codeProvider = new CSharpCodeProvider();

            // setting params for the compiler
            compilerParams = new CompilerParameters();
            compilerParams.GenerateExecutable = false;
            compilerParams.GenerateInMemory = true;
            compilerParams.ReferencedAssemblies.Add("System.dll");
            compilerParams.ReferencedAssemblies.Add("System.Windows.Forms.dll");
            compilerParams.ReferencedAssemblies.Add("System.Core.dll");
            compilerParams.ReferencedAssemblies.Add("System.Drawing.dll");            
        }

        private void tbQuelltext_TextChanged(object sender, EventArgs e)
        {
            pictureBox.Invalidate();
        }

        private void QuelltextKompilierenUndAusführen(object sender, PaintEventArgs e)
        {
            if (bAktualisierungLäuft)
                return;
            bAktualisierungLäuft = true;

            CompilerResults compilerResults =
                codeProvider.CompileAssemblyFromSource(compilerParams, new string[] { tbQuelltext.Text });

            if (!compilerResults.Errors.HasErrors)
            {
                compiledAssembly = compilerResults.CompiledAssembly;
                tbQuelltext.BackColor = Color.White;
            }
            else
            {
                FehlerBehandeln(compilerResults);
            }

            Ausführen(sender, e);

            bAktualisierungLäuft = false;
        }

        private void Ausführen(object sender, PaintEventArgs e)
        {
            object hauptfenster = compiledAssembly.CreateInstance("LiveCodePictureBox.Hauptfenster");
            MethodInfo paintMethod = hauptfenster.GetType().GetMethod("pictureBox_Paint");
            paintMethod.Invoke(hauptfenster, new object[] {sender, e});
        }

        private void FehlerBehandeln(CompilerResults compilerResults)
        {
            tbFehler.Text = "";
            foreach (CompilerError ce in compilerResults.Errors)
            {
                tbFehler.Text += ce.Line + ": " + ce.ErrorText;
                tbQuelltext.BackColor = Color.OrangeRed;
            }
        }

        private void pictureBox_Paint_Proxy(object sender, PaintEventArgs e)
        {
            QuelltextKompilierenUndAusführen(sender, e);            
        } 

        private void HauptfensterProxy_Shown(object sender, EventArgs e)
        {
            tbQuelltext.Focus();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            pictureBox.Invalidate();
        }
    }
}
